{{ define "footer" }}

      </div> <!-- id=main -->
    </div>
    <footer>
      <div class="container">
          <p class="text-muted text-center">Brought to you with <span style="color:#ef5e21">&hearts;</span> by ThousandEyes</p>
      </div>
    </footer>
  </body>
</html>

{{ end }}
