{{ define "events" }}

<div class="col-md-12">
  <div class="card card-default">
    <!-- Default card contents -->
    <div class="card-header">
      Event History
    </div>

    <div class="card-body event-log">
    </div>

  </div>
</div>
{{ end }}
